import { useCallback, useEffect, useRef, useState } from 'react';
var DEFAULT_TRESHOLD = 150;
var stubFunction = function () { return undefined; };
export function useInfiniteScroll(_a) {
    var loading = _a.loading, hasNextPage = _a.hasNextPage, onLoadMore = _a.onLoadMore, _b = _a.threshold, threshold = _b === void 0 ? DEFAULT_TRESHOLD : _b, withPrefetchedData = _a.withPrefetchedData;
    var ref = useRef(null);
    var _c = useState(false), scrollable = _c[0], setScrollable = _c[1];
    var _d = useState(0), scrollbarWidth = _d[0], setScrollWidth = _d[1];
    var _e = useState(0), tableMaxHeight = _e[0], setTableMaxHeight = _e[1];
    var loadMoreItems = loading || !hasNextPage ? stubFunction : onLoadMore;
    var showLoadMore = !loading && !scrollable && hasNextPage;
    var handleScroll = useCallback(function () {
        var parent = ref.current;
        if (parent &&
            parent.scrollTop + parent.clientHeight >= parent.scrollHeight - threshold) {
            loadMoreItems();
        }
    }, [loadMoreItems, threshold]);
    useEffect(function () {
        var tbody = ref.current;
        if (tbody) {
            setTableMaxHeight(calcMaxTBodyHeight(tbody));
            tbody.addEventListener('scroll', handleScroll);
            return function () {
                tbody.removeEventListener('scroll', handleScroll);
            };
        }
    }, [handleScroll]);
    useEffect(function () {
        var tbody = ref.current;
        if (!loading && tbody) {
            if (tableMaxHeight > 0) {
                var tableInnerHeight = calcTBodyInnerHeight(tbody.lastElementChild);
                var scrollableTBody = tableInnerHeight > tableMaxHeight;
                if (scrollableTBody &&
                    tableInnerHeight - tableMaxHeight <= getHeight(tbody.lastElementChild)) {
                    loadMoreItems();
                }
                setScrollable(scrollableTBody);
            }
            else if (!withPrefetchedData) {
                // initial fetch
                loadMoreItems();
            }
            var width = getScrollbarWidth(tbody);
            setScrollWidth(width);
        }
    }, [loading, tableMaxHeight, scrollable, withPrefetchedData, loadMoreItems]);
    return {
        ref: ref,
        tableMaxHeight: tableMaxHeight,
        showLoadMore: showLoadMore,
        scrollbarWidth: scrollbarWidth,
    };
}
//
var getScrollbarWidth = function (tbody) {
    return tbody.clientWidth > 0 ? tbody.offsetWidth - tbody.clientWidth : 0;
};
var getHeight = function (element) {
    return (element === null || element === void 0 ? void 0 : element.clientHeight) || 0;
};
var calcMaxTBodyHeight = function (tbody) {
    var _a;
    var wrapperHeight = getHeight((_a = tbody === null || tbody === void 0 ? void 0 : tbody.parentElement) === null || _a === void 0 ? void 0 : _a.parentElement);
    var headerHeight = getHeight(tbody === null || tbody === void 0 ? void 0 : tbody.previousElementSibling);
    return wrapperHeight - headerHeight;
};
var calcTBodyInnerHeight = function (lastTr) {
    if (lastTr) {
        var lastTrOffset = (lastTr === null || lastTr === void 0 ? void 0 : lastTr.offsetTop) || 0;
        var lastTrHeight = getHeight(lastTr);
        return lastTrOffset + lastTrHeight;
    }
    return 0;
};
